# preset selector for 3dview grid

bl_info = {
		"name": "Grid Presets",
		"blender": (2, 7, 3),
		"version": (0, 0, 1),
		"category": "3D View",
};

import bpy;
import bgl;
import blf;
import math;
from mathutils import *;
from bpy_extras.view3d_utils import *;

grid_preset_enum_items = (
	("1", "1", ""),
	("2", "2", ""),
	("4", "4", ""),
	("8", "8", ""),
	("16", "16", ""),
	("32", "32", ""),
	("64", "64", ""),
	("128", "128", ""),
	("256", "256", ""),
	("512", "512", ""),
	("1024", "1024", ""),
	("2048", "2048", ""),
);

def draw_callback(self, context):
	scale = context.space_data.grid_scale;

	x = 100;
	y = 100;
	strc = "%d" % scale;
	blf.position(0, x, y, 0);
	blf.draw(0, strc);

def update_all_spacevew3d(context, value):
	scale = 1.0;
	try:
		scale = float(value);
	except:
		pass;
	for area in context.screen.areas:
		if(area.type != "VIEW_3D"):
			continue;
		for space in area.spaces:
			if(space.type != "VIEW_3D"):
				continue;
			space.grid_scale = scale;

def grid_preset_enum_update(self, context):
	update_all_spacevew3d(context, self.grid_preset_enum);

def enum_val(cur, idx, prev):
	if(idx >= 0):
		return grid_preset_enum_items[idx][0];

	l = len(grid_preset_enum_items);
	idx = [i for i, r in enumerate(grid_preset_enum_items) if r[0] == cur][0];
	if(prev):
		idx -= 1;
	else:
		idx += 1;
	return grid_preset_enum_items[idx % l][0];

class GridPresetSwitch(bpy.types.Operator):
	bl_idname = "view3d.grid_preset_switch";
	bl_label = "Grid Preset Switch";

	draw_callback_id = None;
	disable = bpy.props.BoolProperty(default=False);
	toggle = bpy.props.BoolProperty(default=False);
	preset_idx = bpy.props.IntProperty();
	switch_prev = bpy.props.BoolProperty(default=False);
	switch_next = bpy.props.BoolProperty(default=False);

	def execute(self, context):
		if(self.toggle):
			self.disable = not self.disable;

		if(self.disable):
			if(GridPresetSwitch.draw_callback_id):
				bpy.types.SpaceView3D.draw_handler_remove(GridPresetSwitch.draw_callback_id,
						"WINDOW");
				GridPresetSwitch.draw_callback_id = None;
		elif(not GridPresetSwitch.draw_callback_id):
			GridPresetSwitch.draw_callback_id = bpy.types.SpaceView3D.draw_handler_add(
					draw_callback, (self, context), "WINDOW", "POST_PIXEL");

		curval = bpy.data.window_managers[0].grid_preset_enum;
		newval = curval;
		if(not self.disable):
			if(self.switch_next):
				newval = enum_val(curval, -1, False);
			elif(self.switch_prev):
				newval = enum_val(curval, -1, True);
			else:
				newval = enum_val(curval, self.preset_idx, False);
		bpy.data.window_managers[0].grid_preset_enum = newval;

		if context.area:
			context.area.tag_redraw();

		return {"FINISHED"};

class GridPresetSwitchPrev(bpy.types.Operator):
	bl_idname = "view3d.grid_preset_switch_prev";
	bl_label = "Grid Preset Switch Previous";
	def execute(self, context):
		return bpy.ops.view3d.grid_preset_switch(switch_prev=True);

class GridPresetSwitchNext(bpy.types.Operator):
	bl_idname = "view3d.grid_preset_switch_next";
	bl_label = "Grid Preset Switch Next";
	def execute(self, context):
		return bpy.ops.view3d.grid_preset_switch(switch_next=True);

def menu_func(self, context):
	row = self.layout.row();
	row.operator(GridPresetSwitch.bl_idname).toggle=True;
	row.prop(bpy.data.window_managers[0], "grid_preset_enum", text="");
	row = self.layout.row();
	row.operator(GridPresetSwitch.bl_idname, text="Prev").switch_prev=True;
	row.operator(GridPresetSwitch.bl_idname, text="Next").switch_next=True;

addon_keymaps = [];

def register():
	bpy.types.WindowManager.grid_preset_enum = bpy.props.EnumProperty(
			name="Grid Preset",
			items=grid_preset_enum_items,
			options={"SKIP_SAVE"},
			update=grid_preset_enum_update
			);

	bpy.utils.register_class(GridPresetSwitch);
	bpy.utils.register_class(GridPresetSwitchPrev);
	bpy.utils.register_class(GridPresetSwitchNext);

	bpy.types.VIEW3D_PT_view3d_display.append(menu_func);

	# keys
	global addon_keymaps;
	wm = bpy.context.window_manager;
	km = wm.keyconfigs.addon.keymaps.new(name="3D View", space_type="VIEW_3D");
	kmi = km.keymap_items.new(GridPresetSwitchPrev.bl_idname, "LEFT_BRACKET", "RELEASE");
	addon_keymaps.append((km, kmi));
	kmi = km.keymap_items.new(GridPresetSwitchNext.bl_idname, "RIGHT_BRACKET", "RELEASE");
	addon_keymaps.append((km, kmi));

def unregister():
	global addon_keymaps;
	for km, kmi in addon_keymaps:
		km.keymap_items.remove(kmi);
	addon_keymaps.clear();

	bpy.utils.unregister_class(GridPresetSwitch);
	bpy.utils.unregister_class(GridPresetSwitchPrev);
	bpy.utils.unregister_class(GridPresetSwitchNext);
	bpy.types.VIEW3D_PT_view3d_display.remove(menu_func);

	if(GridPresetSwitch.draw_callback_id):
		bpy.types.SpaceView3D.draw_handler_remove(GridPresetSwitch.draw_callback_id, "WINDOW");

	del bpy.types.WindowManager.grid_preset_enum;


if __name__ == "__main__":
	register();
